package com.yupi.springbootinit.controller;

import com.alibaba.excel.util.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yupi.springbootinit.common.BaseResponse;
import com.yupi.springbootinit.common.ErrorCode;
import com.yupi.springbootinit.common.ResultUtils;
import com.yupi.springbootinit.exception.BusinessException;
import com.yupi.springbootinit.mapper.commentMapper;
import com.yupi.springbootinit.model.dto.Comment.CommentAddRequest;
import com.yupi.springbootinit.model.entity.Comment;
import com.yupi.springbootinit.model.entity.Post;
import com.yupi.springbootinit.model.entity.User;
import com.yupi.springbootinit.model.vo.CommentVO;
import com.yupi.springbootinit.service.PostService;
import com.yupi.springbootinit.service.UserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import com.yupi.springbootinit.service.commentService;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 帖子评论
 *
 */
@RestController
@RequestMapping("/comment")
@Slf4j
public class commentController {
    /**
     * 获取指定帖子的评论详情
     */
    @Resource
    private commentService  commentService;
    @Resource
    private UserService userService;
    @Resource
    private commentMapper commentMapper;
    @Resource
    private PostService postService;
//    @GetMapping("/comment")
//    public void pingController(HttpServletRequest request){
//        /**
//         * 返回评论详情
//         * 1.返回评论人信息（头像名称）
//         * 2.返回评论内容
//         */
//        User loginUser = userService.getLoginUser(request);
//
//
//    }
    /**
     * 根据帖子ID获取评论列表
     */
    @GetMapping("/comments")
    public BaseResponse<List<CommentVO>> getCommentListByPostId(@RequestParam("postId") Long postId) {
        if (postId == null || postId <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "帖子ID无效");
        }

        // 1. 查询该帖子下的未删除评论
        QueryWrapper<Comment> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("postId", postId);
        queryWrapper.eq("isDelete", false);
        queryWrapper.orderByDesc("createTime");
        List<Comment> commentList = commentService.list(queryWrapper);

        // 2. 提取所有非空的 userId，避免 N+1 查询
        Set<Long> userIdSet = commentList.stream()
                .map(Comment::getUserId)
                .filter(userId -> userId != null) // 过滤掉空的userId
                .collect(Collectors.toSet());

        // 创建用户ID到用户对象的映射
        Map<Long, User> userMap;
        if (!userIdSet.isEmpty()) { // 只有在有用户ID时才查询
            userMap = userService.listByIds(userIdSet).stream()
                    .collect(Collectors.toMap(User::getId, user -> user, (existing, replacement) -> existing));
        } else {
            userMap = new HashMap<>();
        }

        // 3. 组装返回 VO
        List<CommentVO> commentVOList = commentList.stream().map(comment -> {
            CommentVO vo = new CommentVO();
            vo.setContent(comment.getContent());
            vo.setCreateTime(comment.getCreateTime());

            // 处理用户信息，包括空userId的情况
            Long userId = comment.getUserId();
            if (userId != null) {
                User user = userMap.get(userId);
                if (user != null) {
                    vo.setUserName(user.getUserName());
                    vo.setUserAvatar(user.getUserAvatar());
                } else {
                    // 用户ID存在但未找到用户
                    vo.setUserName("未知用户");
                    vo.setUserAvatar(""); // 设置默认头像或留空
                }
            } else {
                // userId为空的情况
                vo.setUserName("匿名用户");
                vo.setUserAvatar(""); // 设置默认头像或留空
            }

            return vo;
        }).collect(Collectors.toList());

        return ResultUtils.success(commentVOList);
    }

    /**
     * 发布评论
     */
    @PostMapping("/sendcomment")
    public BaseResponse<Long> addComment(@RequestBody CommentAddRequest commentAddRequest, HttpServletRequest request) {
//        BaseResponse baseResponse = new BaseResponse();
//        baseResponse.setCode();
//        baseResponse.setData();
//        baseResponse.setMessage();
//        return baseResponse;
        // 1. 参数校验
        if (commentAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "请求参数为空");
        }

        Long postId = commentAddRequest.getPostId();
        String content = commentAddRequest.getContent();

        // 校验帖子ID
        if (postId == null || postId <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "帖子ID无效");
        }

        // 校验评论内容
        if (StringUtils.isBlank(content)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "评论内容不能为空");
        }
        if (content.length() > 1000) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "评论内容过长");
        }

        // 2. 获取当前登录用户
        User loginUser = userService.getLoginUser(request);
        if (loginUser == null) {
            throw new BusinessException(ErrorCode.NOT_LOGIN_ERROR);
        }

        // 3. 校验帖子是否存在
        Post post = postService.getById(postId);
        if (post == null || Boolean.TRUE.equals(post.getIsDelete())) {
            throw new BusinessException(ErrorCode.NOT_FOUND_ERROR, "帖子不存在");
        }

        // 4. 创建评论对象
        Comment comment = new Comment();
        comment.setPostId(postId);
        comment.setContent(content);
        comment.setUserId(loginUser.getId());
        comment.setIsDelete(0);

        // 6. 保存评论
        boolean result = commentService.save(comment);
        if (!result) {
            throw new BusinessException(Erro rCode.SYSTEM_ERROR, "评论发布失败");
        }


        // 8. 返回评论ID
        return ResultUtils.success(comment.getId());
    }




}
